#include "vera_exp_task.h"
#include "vera_directc.h"

static VI_ExpTask *ExpTaskHandle = 0;

/*********** C calling Vera***************/

/*
  Here are the steps for calling Vera exported tasks.

  - Vera: Vera calls the C program so that it calls exported Vera task.
  - C   : Call vera_GetExportMethods() and grab all the exported tasks from Vera.
  - C   : Call NewTask and create a handle for the task returned by GetExportMethods()
  - C   : Call SetArgument() task to set the arguments for calling Vera exported tasks. 
  Use the handle returned by NewTask for this.      
  - C   : Call CallTask() functions with pointer to exported Vera task, sync type, call 
  back function name and void pointer as the arguments.
  - C   : call back function will be called under the hood. 
*/

static void CallBackFunc( void *cdata )
{
  VeraTask	task = (VeraTask) cdata;
  VI_ValueP	vp;
  int		ival;

  printf("C: In CallBackFunc, VeraExportTask must be done\n");
}


void c_call_vera(void)
{
  VeraTaskDecl task_decl  ;
  VeraTask     TaskHandle ;
  VI_ArgType   args[2];
  VI_ValueP    vp;
  VI_VecVal    vec;
  int		ival, x, z;


  if (!ExpTaskHandle)
    ExpTaskHandle = vera_GetExportMethods();
  
  /* First argument  (bit vector of 8 bits) */
  args[0].type  = VI_VECTOR;
  args[0].isVar = 0;
  args[0].size  = 8;
  
  /* Second argument  (var integer) */
  args[1].type  = VI_INT;
  args[1].isVar = 1;
  args[1].size  = 32;

  task_decl  = (*(ExpTaskHandle->GetExportTask))("VeraExportTask", 2, args);
  TaskHandle = (*(ExpTaskHandle->NewTask))(task_decl);

  x = 100;
  vec.aval  = x;
  vec.bval  = 0;
  vp.vecVal = &vec;
  (*(ExpTaskHandle->SetArgument))(TaskHandle,0,vp); /* Set argument 0 (bit vector) */

  z = 1;
  vec.aval = z;
  vec.bval = 0;
  vp.vecVal = &vec;
  (*(ExpTaskHandle->SetArgument))(TaskHandle,1,vp); /* Set argument 1 (var integer) */

  printf("C: call back to Vera with (%d, %d)\n", x, z);

  (*(ExpTaskHandle->CallTask))(TaskHandle, 1, CallBackFunc, TaskHandle); /* Call the Vera exported task*/

  printf("C: returned from VeraExportTask\n");

  vp.intVal = &ival;
  if( (*(ExpTaskHandle->FetchArgument))( TaskHandle, 1, vp ) )
    printf("C: FetchArgument on 2nd argument failed\n" );
  else
    printf("C: Vera's task var argument is %0d\n", ival);

  /* Don't delete the handle - save it for the next call 
     (*(ExpTaskHandle->DeleteTask))(TaskHandle);  // Delete the handle
  */
} // c_call_vera



/********  Vera calling C ********/

void c_task(vec32 *bit_vec)
{
  printf("C: c_task in=%d", bit_vec->d);
  bit_vec->d = 42;
  printf(", out=%d\n", bit_vec->d);

  c_call_vera();
} // c_task
